package simplejca;

import gov.aps.jca.*;
import gov.aps.jca.dbr.*;

/**
 * Gets a process variable
 */
public class SimpleJCAGet1 
{
  private static final double TIMEOUT=1.0;

  double timeout=TIMEOUT;
  String name=null;
  String [] value=null;
  boolean pvSpecified=false;

  /**
   * Default constructor
   */
  public SimpleJCAGet1()
  {
  }

  /**
   * Main entry point
   * @param args arguments
   */
  public static void main(String[] args)
  {
    SimpleJCAGet1 simpleJCAGet1 = new SimpleJCAGet1();
    JCALibrary jca=null;
    Context ctxt=null;
    Channel chan=null;
    
  // Parse the command line
    if(!simpleJCAGet1.parseCommand(args)) System.exit(1);
    if(!simpleJCAGet1.pvSpecified) {
      System.err.println("No PV specified\n");
      System.exit(1);
    }

  // Initialize and search
    try {
      // Get the JCALibrary instance
      jca=JCALibrary.getInstance();
      
      // Create a single threaded context with default configuration values
      // This is for JCA
      // ctxt=jca.createContext(JCALibrary.JNI_SINGLE_THREADED);
      // This is for CAJ
      ctxt=jca.createContext("com.cosylab.epics.caj.CAJContext");
      
      // Search
      chan=ctxt.createChannel(simpleJCAGet1.name);
      
      // Wait for search
      ctxt.pendIO(simpleJCAGet1.timeout);
    
    } catch(Exception ex) {
      System.err.println("Search failed for " + simpleJCAGet1.name + ":\n" + ex);
      System.exit(1);
    }
    
    // Get the first value as a String
    try {
      // Get the value
      simpleJCAGet1.value=((STRING)chan.get(DBRType.STRING,1)).getStringValue();

      // Wait for the get
      ctxt.pendIO(simpleJCAGet1.timeout);
      
      // Print the value
      System.out.println("The value of " + simpleJCAGet1.name + " is " +
       simpleJCAGet1.value[0]);
       
    } catch(Exception ex) {
      System.err.println("Get failed for " + simpleJCAGet1.name + ":\n" + ex);
      System.exit(1);
    }
    
    // Clean up
    try {
      // Clear the channel
      chan.destroy();
      
      // Destroy the context
      ctxt.destroy();
      
    } catch(Exception ex) {
      System.err.println("Clean up failed for " + simpleJCAGet1.name + ":\n" +
       ex);
      System.exit(1);
    }

    System.exit(0);
    
  }
  
  /**
   * Parse the command line 
   * 
   * @param args
   * @return success or failure
   */
  private boolean parseCommand(String[] args)
  {
    int i;
  
    for(i=0; i < args.length; i++) {
      if (args[i].startsWith("-")) {
        switch(args[i].charAt(1)) {
        case 'h':
          usage();
          System.exit(0);
        case 't':
          try {
            timeout=Double.valueOf(args[++i]).doubleValue();
          } catch(NumberFormatException ex)  {
            System.err.println("\n\nInvalid timeout: " + args[i]);
            usage();
            return false;
          }
          break;
        default:
          System.err.println("\n\nInvalid option: " + args[i]);
          usage();
          return false;
        }
      } else {
        if(!pvSpecified) {
          name=args[i];
          pvSpecified=true;
        } else {
          System.err.println("\n\nInvalid option: " + args[i]);
          usage();
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Print usage
   */
  private void usage()
  {
    System.err.println("\nUsage: java SimpleJCAGet1 [Options] pvname\n" +
     "  Connects to pvname and gets the value\n" +
     "\n" +
     "  Options:\n" +
     "    -h help      This message\n" +
     "    -t float     Timeout in seconds (Default: " + TIMEOUT + ")\n");
  }

}
